import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';
import { menu } from './menu';

const csvUrl = [
  'https://gist.githubusercontent.com/',
  'curran/', // User
  'a08a1080b88344b0c8a7/', // Id of the Gist
  'raw/0e7a9b0a5d22642a06d3d5b9bcbad9890c8ee534/', // commit
  'iris.csv', // File name
].join('');
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

const width = window.innerWidth;
const height = window.innerHeight;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const menuContainer = select('body')
  .append('div')
  .attr('class', 'menu-container');

const xMenu = menuContainer.append('div');
const yMenu = menuContainer.append('div');

const main = async () => {
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.petal_width)
    .yValue((d) => d.species)
  	.xType('linear')
  	.yType('band')
  	.xColor('black')
    .margin({
      top: 20,
      right: 20,
      bottom: 40,
      left: 110,
    })
    .radius(5);

  svg.call(plot);

  const options = [
    { column: 'petal_width', text: 'Petal Width (linear)', type: 'linear' },
    { column: 'sepal_width', text: 'Sepal Width (linear)', type: 'linear' },
    { column: 'petal_length', text: 'Petal Length (linear)', type: 'linear' },
    { column: 'sepal_length', text: 'Sepal Length (linear)', type: 'linear' },
    
    { column: 'species', text: 'Species (categorical)', type: 'band' },
    { column: 'petal_width', text: 'Petal Width (square)', type: 'square' },
    { column: 'sepal_width', text: 'Sepal Width (power)', type: 'power' },
    { column: 'petal_length', text: 'Petal Length (square)', type: 'square' },
    { column: 'sepal_length', text: 'Sepal Length (power)', type: 'power' },
    
  ];

  xMenu.call(
    menu()
      .id('x-menu')
      .labelText('X:')
      .options(options)
      .on('change', (val) => {       
        //asign
        svg.call(plot.xType(options[val].type).xValue( d => d[options[val].column] ));
        
        //svg.call(plot.xValue((d) => d[column]).xType());
      })
      .on('x-click', (lbl) => {
        if (plot.xColor() === 'blue') {
          svg.call(plot.xColor('black'));
        } else {
          svg.call(plot.xColor('blue'));
        }
      })
  );
  yMenu.call(
    menu()
      .id('y-menu')
      .labelText('Y:')
      .options(options)
      .on('change', (val) => {
        svg.call(plot.yType(options[val].type).yValue( d => d[options[val].column] ));
        
        //
       // svg.call(plot.yValue((d) => d[column]));
      })
  );
};
main();
